// Simple RFM12B sender program, with ACK and optional encryption
// It initializes the RFM12B radio with optional encryption and passes through any valid messages to the serial port
// felix@lowpowerlab.com
#include <RFM12B.h>
#include <avr/sleep.h>
// You will need to initialize the radio by telling it what ID it has and what network it's on
// The NodeID takes values from 1-127, 0 is reserved for sending broadcast messages (send to all nodes)
// The Network ID takes values from 0-255
// By default the SPI-SS line used is D10 on Atmega328. You can change it by calling .SetCS(pin) where pin can be {8,9,10}
#define NODEID        2  //network ID used for this unit
#define NETWORKID    99  //the network ID we are on
#define GATEWAYID     1  //the node ID we're sending to
#define ACK_TIME     900 // # of ms to wait for an ack
#define forward_pin  18
#define back_pin     7
#define right_pin    6
#define left_pin     5
#define up_pin       4
#define down_pin     3
#define worklight_pin 17

int interPacketDelay = 50; //wait this many ms between sending packets
char input = 's';
// Need an instance of the Radio Module
RFM12B radio;
byte sendSize = 0;
char payload[] = "s";
bool requestACK = false;
void setup()
{
  
  Serial.begin(115200);
  delay(3000);
  Serial.println("Starting...");
  pinMode (forward_pin,INPUT);
  pinMode (back_pin,INPUT);
  pinMode (right_pin,INPUT);
  pinMode (left_pin,INPUT);
  pinMode (up_pin,INPUT);
  pinMode (down_pin,INPUT);
  pinMode (worklight_pin,INPUT);
  digitalWrite(forward_pin,1);
  digitalWrite(back_pin,1);
  digitalWrite(right_pin,1);
  digitalWrite(left_pin,1);
  digitalWrite(up_pin,1);
  digitalWrite(down_pin,1);
  digitalWrite(worklight_pin,1);
  radio.Initialize(NODEID, RF12_433MHZ, NETWORKID);
  delay(500);
}
void loop()
{
    if (digitalRead(forward_pin)==LOW) send_rad('f');
      else if (digitalRead(back_pin)==LOW) send_rad('b');
        else if (digitalRead(right_pin)==LOW) send_rad('r');
          else if (digitalRead(left_pin)==LOW) send_rad('l');
            else if (digitalRead(up_pin)==LOW) send_rad('u');
              else if (digitalRead(down_pin)==LOW) send_rad('d');
                else {
                   
                  // Only deal with the worklight if not moving9
                  if (digitalRead(worklight_pin)==HIGH) send_rad('w');
                    else send_rad('x');
                  delay(interPacketDelay);
                  send_rad('s');
                  Serial.println(digitalRead(worklight_pin));
                }    
}
void send_rad (char srchar) {
  payload[0] = srchar;
  radio.Send(GATEWAYID, payload, 1, requestACK);
    if (requestACK)   waitForAck();
    sendSize = 1;
    delay(interPacketDelay);
}
// wait a few milliseconds for proper ACK, return true if received
static bool waitForAck() {
  long now = millis();
  while (millis() - now <= ACK_TIME)
    if (radio.ACKReceived(GATEWAYID))
      return true;
  return false;
}

