/*
  Moteino Drive Gear Drive with Cab
  Radio Command Control forward back right left stop
  "f":  forward tacks
  "b":  backward tracks
  "r":  tracks turn right
  "l":  tracks turn left
  "s":  tracks stop
  "u":  hook up
  "d":  hook down
  "p":  hook auto down then up
  "w":  work light On
  "x":  work light Off
  
*/
String track_cmds ="fbrlsudpwx";
#include <RFM12B.h>
#define NODEID           1  //network ID used for this unit
#define NETWORKID       99  //the network ID we are on
// Need an instance of the Radio Module
RFM12B radio; 
int m2h = 6;   //R Left Track
int m2l = 7;   //B Left Track
int m0h = 8;   //R Right Track
int m0l = 9;   //B Right Track
int en  = 5;   //Enable
int m4h = 15;  // boom drive pin
int m4l = 16;  // boom drive pin
int worklight = 17;  // Front Work Light LED Pin

int kickstarton = 1500; //kick start cycle on time
int kickstarttime = 7; //kick start duration on time
int fwdon = 1000;
int fwdtime = 20;
int bwdon;
int bwdtime = 20;
int bwdshift = 0;
int right180time = 1620;
int right90time = 810;
int righttime = 20;
int righton = 1000;
int left180time = 1596;
int left90time = 798;
int lefttime = 20;
int lefton = 1000;
int cyclewidth = 2000;

int speedup = 112;   //Right track time differntial
int deltime = 1500;
int boomtime = 200;
int boomon = 500;   //380 jerky
int lctr = 0;
int turns =0;
char rec_char;

void setup() { 
  Serial.begin(115200);
  radio.Initialize(NODEID, RF12_433MHZ, NETWORKID);
  // initialize the digital pins as outputs
  pinMode(m2h, OUTPUT);
  pinMode(m2l, OUTPUT);
  pinMode(m0h, OUTPUT);
  pinMode(m0l, OUTPUT);
  pinMode(en,  OUTPUT);
  pinMode(m4h, OUTPUT);
  pinMode(m4l, OUTPUT);
  pinMode(worklight, OUTPUT);
  digitalWrite(m2l, LOW);
  digitalWrite(m2h, LOW);
  digitalWrite(m0l, LOW);
  digitalWrite(m0h, LOW);
  digitalWrite(en,  LOW);      
  digitalWrite(m4h, LOW);
  digitalWrite(m4l, LOW);
  digitalWrite(worklight, LOW);
  bwdon = fwdon-bwdshift;
  randomSeed(3.14159*analogRead(0));
  turns=0;
  rec_char = 's';
  delay(3000);
  Serial.println("Starting...");
}
void loop() {
  if (radio.ReceiveComplete())  {
    if (radio.CRCPass())  {
      Serial.print('[');Serial.print(radio.GetSender());Serial.print("] ");
        rec_char = radio.Data[0];
        Serial.println(rec_char);
    }
  }
  switch (track_cmds.indexOf(rec_char)+1) {
  case 1:   //"f": forward tracks
      gofwd (kickstarttime,kickstarton);
      gofwd (fwdtime,fwdon);
      break;
   case 2:   //"b": backward tracks
      gobwd (kickstarttime,kickstarton);
      gobwd (bwdtime,bwdon);
      break;
   case 3:   //"r":  tracks turn right
      goright (kickstarttime,kickstarton);
      goright (righttime,righton); 
      break;
   case 4:   //"l":  tracks turn left
      goleft (kickstarttime,kickstarton);
      goleft (lefttime,lefton);
      break;
   case 5:    //"s":  tracks stop
      break; 
   case 6:   //"u":  boom up
      boomup (kickstarttime,kickstarton);
      boomup (boomtime,boomon);
      break;
   case 7:   //"d":  boom down
      boomdown (kickstarttime,kickstarton);
      boomdown (boomtime,boomon);
      break;
   case 8:   //"p":  pickup: boom down, boom up
      boomdown (kickstarttime,kickstarton);
      boomdown (830,boomon);
      delay (800);
      boomup (kickstarttime,kickstarton);
      boomup (1100,boomon);
      rec_char = 's';
      break;
   case 9:   //"w":  Work Light On
      digitalWrite(worklight, HIGH);
      break;
   case 10:   //"x":  Work Light Off
      digitalWrite(worklight, LOW);
      break;	  
   default:
       
       break;
  }
}
void gofwd(int fcnt,int fcycle) {
   int icnt;
   int totcycle;
   icnt = 0;
   totcycle = fcycle+speedup;
   digitalWrite(en, HIGH);
   while (icnt < fcnt)
  {
    digitalWrite(m2h, HIGH);
    digitalWrite(m0h, HIGH);
    delayMicroseconds(fcycle);
    digitalWrite(m2h, LOW);
    delayMicroseconds (speedup);
    digitalWrite(m0h, LOW);
    delayMicroseconds(cyclewidth - totcycle);
    icnt++;
  }
  digitalWrite(en, LOW);
}
void gobwd(int bcnt,int bcycle) {
   int icnt;
   icnt=0;
   digitalWrite(en, HIGH);
   while (icnt < bcnt)
  {
    digitalWrite(m2l, HIGH);
    digitalWrite(m0l, HIGH);
    delayMicroseconds(bcycle); 
    digitalWrite(m2l, LOW);
    digitalWrite(m0l, LOW);
    delayMicroseconds(cyclewidth - bcycle);
    icnt++;
  }
  digitalWrite(en, LOW);
}
void goright(int rcnt,int rcycle) {
   int icnt;
   icnt=0;
   digitalWrite(en, HIGH);
   while (icnt < rcnt)
  {
    digitalWrite(m2h, HIGH);
    digitalWrite(m0l, HIGH);
    delayMicroseconds(rcycle); 
    digitalWrite(m2h, LOW);
    digitalWrite(m0l, LOW);
    delayMicroseconds(cyclewidth - rcycle);
    icnt++;
  }
  digitalWrite(en, LOW);
}
void goleft(int lcnt,int lcycle) {
   int icnt;
   icnt=0;
   digitalWrite(en, HIGH);
   while (icnt < lcnt)
  {
    digitalWrite(m2l, HIGH);
    digitalWrite(m0h, HIGH);
    delayMicroseconds(lcycle); 
    digitalWrite(m2l, LOW);
    digitalWrite(m0h, LOW);
    delayMicroseconds(cyclewidth - lcycle);
    icnt++;
  }
  digitalWrite(en, LOW);
}
void boomup(int bmcnt,int bmcycle) {
   int icnt;
   icnt=0;
   digitalWrite(en, HIGH);
   while (icnt < bmcnt)
  {
    digitalWrite(m4l, HIGH);
    delayMicroseconds(bmcycle); 
    digitalWrite(m4l, LOW);
    delayMicroseconds(cyclewidth - bmcycle);
    icnt++;
  }
  digitalWrite(en, LOW);
}
void boomdown(int bmcnt,int bmcycle) {
   int icnt;
   icnt=0;
   digitalWrite(en, HIGH);
   while (icnt < bmcnt)
  {
    digitalWrite(m4h, HIGH);
    delayMicroseconds(bmcycle); 
    digitalWrite(m4h, LOW);
    delayMicroseconds(cyclewidth - bmcycle);
    icnt++;
  }
  digitalWrite(en, LOW);
}
